START TRANSACTION;


CREATE TABLE "params_str" (
  "paramname" CHARACTER LARGE OBJECT NOT NULL,
  "value"     CHARACTER LARGE OBJECT,
	"prob"      DOUBLE     NOT NULL
);

CREATE TABLE "gtaa_obj_dict" (
	"idstr" CHARACTER LARGE OBJECT NOT NULL,
	"id"    INTEGER       NOT NULL,
	"type"  CHARACTER LARGE OBJECT NOT NULL,
	"prob"  FLOAT(51)     NOT NULL,
	CONSTRAINT "gtaa_obj_dict_id_pkey" PRIMARY KEY ("id"),
	CONSTRAINT "gtaa_obj_dict_idstr_unique" UNIQUE ("idstr")
);

CREATE TABLE "gtaa_materialized_1" (
	"id"    INTEGER       NOT NULL,
	"pid"   CHARACTER LARGE OBJECT NOT NULL,
	"level" INTEGER,
	"class" CHARACTER LARGE OBJECT NOT NULL,
	"json"  CHARACTER LARGE OBJECT,
	"xml"   CHARACTER LARGE OBJECT,
	CONSTRAINT "gtaa_materialized_1_id_pkey" PRIMARY KEY ("id"),
	CONSTRAINT "gtaa_materialized_1_pid_unique" UNIQUE ("pid")
);

CREATE TABLE "gtaa_all_termdict_snowball" (
	"termid" INTEGER       NOT NULL,
	"term"   CHARACTER LARGE OBJECT NOT NULL,
	"prob"   FLOAT(51)     NOT NULL,
	CONSTRAINT "gtaa_all_termdict_snowball_termid_pkey" PRIMARY KEY ("termid"),
	CONSTRAINT "gtaa_all_termdict_snowball_term_unique" UNIQUE ("term")
);

CREATE TABLE "gtaa_all_termdict_unstemmed" (
	"termid" INTEGER       NOT NULL,
	"term"   CHARACTER LARGE OBJECT NOT NULL,
	"sb"     INTEGER       NOT NULL,
	"prob"   DOUBLE        NOT NULL,
	CONSTRAINT "gtaa_all_termdict_unstemmed_termid_pkey" PRIMARY KEY ("termid"),
	CONSTRAINT "gtaa_all_termdict_unstemmed_term_unique" UNIQUE ("term"),
	CONSTRAINT "gtaa_all_termdict_unstemmed_sb_fkey" FOREIGN KEY ("sb") REFERENCES "gtaa_all_termdict_snowball" ("termid")
);


CREATE TABLE "gtaa_all_tf_bm25_unstemmed" (
	"termid" INTEGER       NOT NULL,
	"objid"  INTEGER       NOT NULL,
	"prob"   DOUBLE        NOT NULL,
	CONSTRAINT "gtaa_all_tf_bm25_unstemmed_objid_fkey" FOREIGN KEY ("objid") REFERENCES "gtaa_obj_dict" ("id"),
	CONSTRAINT "gtaa_all_tf_bm25_unstemmed_termid_fkey" FOREIGN KEY ("termid") REFERENCES "gtaa_all_termdict_unstemmed" ("termid")
);

CREATE TABLE "gtaa_all_idf_bm25_unstemmed" (
	"termid" INTEGER       NOT NULL,
	"prob"   DOUBLE        NOT NULL,
	CONSTRAINT "gtaa_all_idf_bm25_unstemmed_termid_fkey" FOREIGN KEY ("termid") REFERENCES "gtaa_all_termdict_unstemmed" ("termid")
);


CREATE TABLE "gtaa_obj_tree" (
	"tree"    CHARACTER LARGE OBJECT,
	"subject" INTEGER       NOT NULL,
	"pre"     BIGINT,
	"post"    BIGINT,
	"size"    BIGINT,
	"level"   TINYINT,
	"prob"    DOUBLE        NOT NULL,
	CONSTRAINT "gtaa_obj_tree_pre_unique" UNIQUE ("pre"),
	CONSTRAINT "gtaa_obj_tree_post_unique" UNIQUE ("post"),
	CONSTRAINT "gtaa_obj_tree_subject_fkey" FOREIGN KEY ("subject") REFERENCES "gtaa_obj_dict" ("id")
);

CREATE TABLE "gtaa_obj_region" (
	"subject"   INTEGER       NOT NULL,
	"reference" INTEGER,
	"line"      CHARACTER LARGE OBJECT NOT NULL,
	"startpos"  BIGINT        NOT NULL,
	"endpos"    BIGINT        NOT NULL,
	"prob"      DOUBLE        NOT NULL,
	CONSTRAINT "gtaa_obj_region_subject_fkey" FOREIGN KEY ("subject") REFERENCES "gtaa_obj_dict" ("id")
);

CREATE TABLE "am_obj_dict" (
	"idstr" CHARACTER LARGE OBJECT NOT NULL,
	"id"    INTEGER       NOT NULL,
	"type"  CHARACTER LARGE OBJECT NOT NULL,
	"prob"  FLOAT(51)     NOT NULL,
	CONSTRAINT "am_obj_dict_id_pkey" PRIMARY KEY ("id"),
	CONSTRAINT "am_obj_dict_idstr_unique" UNIQUE ("idstr")
);

CREATE TABLE "am_materialized_1" (
	"id"    INTEGER       NOT NULL,
	"pid"   CHARACTER LARGE OBJECT NOT NULL,
	"level" INTEGER,
	"class" CHARACTER LARGE OBJECT NOT NULL,
	"json"  CHARACTER LARGE OBJECT,
	"xml"   CHARACTER LARGE OBJECT,
	CONSTRAINT "am_materialized_1_id_pkey" PRIMARY KEY ("id"),
	CONSTRAINT "am_materialized_1_pid_unique" UNIQUE ("pid")
);

CREATE TABLE "am_all_termdict_snowball" (
	"termid" INTEGER       NOT NULL,
	"term"   CHARACTER LARGE OBJECT NOT NULL,
	"prob"   FLOAT(51)     NOT NULL,
	CONSTRAINT "am_all_termdict_snowball_termid_pkey" PRIMARY KEY ("termid"),
	CONSTRAINT "am_all_termdict_snowball_term_unique" UNIQUE ("term")
);

CREATE TABLE "am_all_termdict_unstemmed" (
	"termid" INTEGER       NOT NULL,
	"term"   CHARACTER LARGE OBJECT NOT NULL,
	"sb"     INTEGER       NOT NULL,
	"prob"   DOUBLE        NOT NULL,
	CONSTRAINT "am_all_termdict_unstemmed_termid_pkey" PRIMARY KEY ("termid"),
	CONSTRAINT "am_all_termdict_unstemmed_term_unique" UNIQUE ("term"),
	CONSTRAINT "am_all_termdict_unstemmed_sb_fkey" FOREIGN KEY ("sb") REFERENCES "am_all_termdict_snowball" ("termid")
);


CREATE TABLE "am_all_tf_bm25_unstemmed" (
	"termid" INTEGER       NOT NULL,
	"objid"  INTEGER       NOT NULL,
	"prob"   DOUBLE        NOT NULL,
	CONSTRAINT "am_all_tf_bm25_unstemmed_objid_fkey" FOREIGN KEY ("objid") REFERENCES "am_obj_dict" ("id"),
	CONSTRAINT "am_all_tf_bm25_unstemmed_termid_fkey" FOREIGN KEY ("termid") REFERENCES "am_all_termdict_unstemmed" ("termid")
);

CREATE TABLE "am_all_idf_bm25_unstemmed" (
	"termid" INTEGER       NOT NULL,
	"prob"   DOUBLE        NOT NULL,
	CONSTRAINT "am_all_idf_bm25_unstemmed_termid_fkey" FOREIGN KEY ("termid") REFERENCES "am_all_termdict_unstemmed" ("termid")
);


CREATE TABLE "am_obj_tree" (
	"tree"    CHARACTER LARGE OBJECT,
	"subject" INTEGER       NOT NULL,
	"pre"     BIGINT,
	"post"    BIGINT,
	"size"    BIGINT,
	"level"   TINYINT,
	"prob"    DOUBLE        NOT NULL,
	CONSTRAINT "am_obj_tree_pre_unique" UNIQUE ("pre"),
	CONSTRAINT "am_obj_tree_post_unique" UNIQUE ("post"),
	CONSTRAINT "am_obj_tree_subject_fkey" FOREIGN KEY ("subject") REFERENCES "am_obj_dict" ("id")
);

CREATE TABLE "am_obj_region" (
	"subject"   INTEGER       NOT NULL,
	"reference" INTEGER,
	"line"      CHARACTER LARGE OBJECT NOT NULL,
	"startpos"  BIGINT        NOT NULL,
	"endpos"    BIGINT        NOT NULL,
	"prob"      DOUBLE        NOT NULL,
	CONSTRAINT "am_obj_region_subject_fkey" FOREIGN KEY ("subject") REFERENCES "am_obj_dict" ("id")
);


INSERT INTO params_str VALUES ('s_ontologySource_CHUNK','am',1.0);
INSERT INTO params_str VALUES ('s_generator_bag_of_words_ATTRIBUTE_A','http://www.w3.org/2004/02/skos/core#prefLabel',1.0);
INSERT INTO params_str VALUES ('s_generator_bag_of_words_ATTRIBUTE_B','http://www.w3.org/2004/02/skos/core#prefLabel',1.0);
CREATE VIEW s__RESULTVIEW__2084207473789121672 AS SELECT id AS a1, prob AS prob FROM am_obj_dict;
CREATE VIEW s__RESULTVIEW__3744103893006362204 AS SELECT tree AS a1, subject AS a2, pre AS a3, post AS a4, size AS a5, level AS a6, prob AS prob FROM am_obj_tree WHERE tree = 'RDF_properties';
CREATE VIEW s__RESULTVIEW__5907800622365123282 AS SELECT paramName AS a1, value AS a2, prob AS prob FROM params_str WHERE paramName = 's_generator_bag_of_words_ATTRIBUTE_A';
CREATE VIEW s__RESULTVIEW__8689322627111648167 AS SELECT id AS a1, prob AS prob FROM gtaa_obj_dict;
CREATE VIEW s__RESULTVIEW__6885005865828173703 AS SELECT tree AS a1, subject AS a2, pre AS a3, post AS a4, size AS a5, level AS a6, prob AS prob FROM gtaa_obj_tree WHERE tree = 'RDF_properties';
CREATE VIEW s__RESULTVIEW_6439365964835459461 AS SELECT paramName AS a1, value AS a2, prob AS prob FROM params_str WHERE paramName = 's_generator_bag_of_words_ATTRIBUTE_B';
CREATE VIEW s__RESULTVIEW__1163027180486560746 AS SELECT am_obj_dict.idstr AS a1, am_obj_dict.id AS a2, am_obj_dict.type AS a3, s__RESULTVIEW__5907800622365123282.a1 AS a4, s__RESULTVIEW__5907800622365123282.a2 AS a5, am_obj_dict.prob * s__RESULTVIEW__5907800622365123282.prob AS prob FROM am_obj_dict,s__RESULTVIEW__5907800622365123282 WHERE am_obj_dict.idstr = s__RESULTVIEW__5907800622365123282.a2;
CREATE VIEW s__RESULTVIEW__2998122681277138649 AS SELECT gtaa_obj_dict.idstr AS a1, gtaa_obj_dict.id AS a2, gtaa_obj_dict.type AS a3, s__RESULTVIEW_6439365964835459461.a1 AS a4, s__RESULTVIEW_6439365964835459461.a2 AS a5, gtaa_obj_dict.prob * s__RESULTVIEW_6439365964835459461.prob AS prob FROM gtaa_obj_dict,s__RESULTVIEW_6439365964835459461 WHERE gtaa_obj_dict.idstr = s__RESULTVIEW_6439365964835459461.a2;
CREATE VIEW s__RESULTVIEW_2168537811202848609 AS SELECT a2 AS a1, prob AS prob FROM s__RESULTVIEW__1163027180486560746;
CREATE VIEW s__RESULTVIEW__5292357521228590801 AS SELECT a2 AS a1, prob AS prob FROM s__RESULTVIEW__2998122681277138649;
CREATE VIEW s__RESULTVIEW__3252966175416019566 AS SELECT s__RESULTVIEW__3744103893006362204.a1 AS a1, s__RESULTVIEW__3744103893006362204.a2 AS a2, s__RESULTVIEW__3744103893006362204.a3 AS a3, s__RESULTVIEW__3744103893006362204.a4 AS a4, s__RESULTVIEW__3744103893006362204.a5 AS a5, s__RESULTVIEW__3744103893006362204.a6 AS a6, s__RESULTVIEW_2168537811202848609.a1 AS a7, s__RESULTVIEW__3744103893006362204.prob * s__RESULTVIEW_2168537811202848609.prob AS prob FROM s__RESULTVIEW__3744103893006362204,s__RESULTVIEW_2168537811202848609 WHERE s__RESULTVIEW__3744103893006362204.a2 = s__RESULTVIEW_2168537811202848609.a1;
CREATE VIEW s__RESULTVIEW__2785167906442910809 AS SELECT s__RESULTVIEW__6885005865828173703.a1 AS a1, s__RESULTVIEW__6885005865828173703.a2 AS a2, s__RESULTVIEW__6885005865828173703.a3 AS a3, s__RESULTVIEW__6885005865828173703.a4 AS a4, s__RESULTVIEW__6885005865828173703.a5 AS a5, s__RESULTVIEW__6885005865828173703.a6 AS a6, s__RESULTVIEW__5292357521228590801.a1 AS a7, s__RESULTVIEW__6885005865828173703.prob * s__RESULTVIEW__5292357521228590801.prob AS prob FROM s__RESULTVIEW__6885005865828173703,s__RESULTVIEW__5292357521228590801 WHERE s__RESULTVIEW__6885005865828173703.a2 = s__RESULTVIEW__5292357521228590801.a1;
CREATE VIEW s__RESULTVIEW_5617587398064666981 AS SELECT a1 AS a1, a2 AS a2, a3 AS a3, a4 AS a4, a5 AS a5, a6 AS a6, prob AS prob FROM s__RESULTVIEW__3252966175416019566;
CREATE VIEW s__RESULTVIEW__3074746299039760313 AS SELECT a1 AS a1, a2 AS a2, a3 AS a3, a4 AS a4, a5 AS a5, a6 AS a6, prob AS prob FROM s__RESULTVIEW__2785167906442910809;
CREATE VIEW s__RESULTVIEW__1300470491949402385 AS SELECT s__RESULTVIEW__3744103893006362204.a1 AS a1, s__RESULTVIEW__3744103893006362204.a2 AS a2, s__RESULTVIEW__3744103893006362204.a3 AS a3, s__RESULTVIEW__3744103893006362204.a4 AS a4, s__RESULTVIEW__3744103893006362204.a5 AS a5, s__RESULTVIEW__3744103893006362204.a6 AS a6, s__RESULTVIEW_5617587398064666981.a1 AS a7, s__RESULTVIEW_5617587398064666981.a2 AS a8, s__RESULTVIEW_5617587398064666981.a3 AS a9, s__RESULTVIEW_5617587398064666981.a4 AS a10, s__RESULTVIEW_5617587398064666981.a5 AS a11, s__RESULTVIEW_5617587398064666981.a6 AS a12, s__RESULTVIEW__3744103893006362204.prob * s__RESULTVIEW_5617587398064666981.prob AS prob FROM s__RESULTVIEW__3744103893006362204,s__RESULTVIEW_5617587398064666981 WHERE s__RESULTVIEW__3744103893006362204.a3 >= s__RESULTVIEW_5617587398064666981.a3 AND s__RESULTVIEW__3744103893006362204.a3 <= (s__RESULTVIEW_5617587398064666981.a3 + s__RESULTVIEW_5617587398064666981.a5);
CREATE VIEW s__RESULTVIEW_7206289517542240945 AS SELECT s__RESULTVIEW__6885005865828173703.a1 AS a1, s__RESULTVIEW__6885005865828173703.a2 AS a2, s__RESULTVIEW__6885005865828173703.a3 AS a3, s__RESULTVIEW__6885005865828173703.a4 AS a4, s__RESULTVIEW__6885005865828173703.a5 AS a5, s__RESULTVIEW__6885005865828173703.a6 AS a6, s__RESULTVIEW__3074746299039760313.a1 AS a7, s__RESULTVIEW__3074746299039760313.a2 AS a8, s__RESULTVIEW__3074746299039760313.a3 AS a9, s__RESULTVIEW__3074746299039760313.a4 AS a10, s__RESULTVIEW__3074746299039760313.a5 AS a11, s__RESULTVIEW__3074746299039760313.a6 AS a12, s__RESULTVIEW__6885005865828173703.prob * s__RESULTVIEW__3074746299039760313.prob AS prob FROM s__RESULTVIEW__6885005865828173703,s__RESULTVIEW__3074746299039760313 WHERE s__RESULTVIEW__6885005865828173703.a3 >= s__RESULTVIEW__3074746299039760313.a3 AND s__RESULTVIEW__6885005865828173703.a3 <= (s__RESULTVIEW__3074746299039760313.a3 + s__RESULTVIEW__3074746299039760313.a5);
CREATE VIEW s__RESULTVIEW_7233975749376649663 AS SELECT a2 AS a1, 1.0 - prod(1.0 - prob) AS prob FROM s__RESULTVIEW__1300470491949402385 GROUP BY a2;
CREATE VIEW s__RESULTVIEW_1228285373361863827 AS SELECT a2 AS a1, 1.0 - prod(1.0 - prob) AS prob FROM s__RESULTVIEW_7206289517542240945 GROUP BY a2;
CREATE VIEW s__RESULTVIEW_576746916810995434 AS SELECT s__RESULTVIEW_7233975749376649663.a1 AS a1, am_obj_region.subject AS a2, am_obj_region.reference AS a3, am_obj_region.line AS a4, am_obj_region.startpos AS a5, am_obj_region.endpos AS a6, s__RESULTVIEW_7233975749376649663.prob * am_obj_region.prob AS prob FROM s__RESULTVIEW_7233975749376649663,am_obj_region WHERE s__RESULTVIEW_7233975749376649663.a1 = am_obj_region.reference;
CREATE VIEW s__RESULTVIEW__4026962281427654916 AS SELECT s__RESULTVIEW_1228285373361863827.a1 AS a1, gtaa_obj_region.subject AS a2, gtaa_obj_region.reference AS a3, gtaa_obj_region.line AS a4, gtaa_obj_region.startpos AS a5, gtaa_obj_region.endpos AS a6, s__RESULTVIEW_1228285373361863827.prob * gtaa_obj_region.prob AS prob FROM s__RESULTVIEW_1228285373361863827,gtaa_obj_region WHERE s__RESULTVIEW_1228285373361863827.a1 = gtaa_obj_region.reference;
CREATE VIEW s__RESULTVIEW__148095467512199531 AS SELECT s__RESULTVIEW_576746916810995434.a1 AS a1, s__RESULTVIEW_576746916810995434.a2 AS a2, s__RESULTVIEW_576746916810995434.a3 AS a3, s__RESULTVIEW_576746916810995434.a4 AS a4, s__RESULTVIEW_576746916810995434.a5 AS a5, s__RESULTVIEW_576746916810995434.a6 AS a6, am_all_tf_bm25_unstemmed.termID AS a7, am_all_tf_bm25_unstemmed.objID AS a8, s__RESULTVIEW_576746916810995434.prob * am_all_tf_bm25_unstemmed.prob AS prob FROM s__RESULTVIEW_576746916810995434,am_all_tf_bm25_unstemmed WHERE s__RESULTVIEW_576746916810995434.a2 = am_all_tf_bm25_unstemmed.objID;
CREATE VIEW s__RESULTVIEW_3487524704470538157 AS SELECT s__RESULTVIEW__4026962281427654916.a1 AS a1, s__RESULTVIEW__4026962281427654916.a2 AS a2, s__RESULTVIEW__4026962281427654916.a3 AS a3, s__RESULTVIEW__4026962281427654916.a4 AS a4, s__RESULTVIEW__4026962281427654916.a5 AS a5, s__RESULTVIEW__4026962281427654916.a6 AS a6, gtaa_all_tf_bm25_unstemmed.termID AS a7, gtaa_all_tf_bm25_unstemmed.objID AS a8, s__RESULTVIEW__4026962281427654916.prob * gtaa_all_tf_bm25_unstemmed.prob AS prob FROM s__RESULTVIEW__4026962281427654916,gtaa_all_tf_bm25_unstemmed WHERE s__RESULTVIEW__4026962281427654916.a2 = gtaa_all_tf_bm25_unstemmed.objID;
CREATE VIEW s__RESULTVIEW_8636114359657583065 AS SELECT s__RESULTVIEW__148095467512199531.a1 AS a1, s__RESULTVIEW__148095467512199531.a2 AS a2, s__RESULTVIEW__148095467512199531.a3 AS a3, s__RESULTVIEW__148095467512199531.a4 AS a4, s__RESULTVIEW__148095467512199531.a5 AS a5, s__RESULTVIEW__148095467512199531.a6 AS a6, s__RESULTVIEW__148095467512199531.a7 AS a7, s__RESULTVIEW__148095467512199531.a8 AS a8, s__RESULTVIEW__2084207473789121672.a1 AS a9, s__RESULTVIEW__148095467512199531.prob * s__RESULTVIEW__2084207473789121672.prob AS prob FROM s__RESULTVIEW__148095467512199531,s__RESULTVIEW__2084207473789121672 WHERE s__RESULTVIEW__148095467512199531.a2 = s__RESULTVIEW__2084207473789121672.a1;
CREATE VIEW s__RESULTVIEW_4914011101465958082 AS SELECT s__RESULTVIEW_3487524704470538157.a1 AS a1, s__RESULTVIEW_3487524704470538157.a2 AS a2, s__RESULTVIEW_3487524704470538157.a3 AS a3, s__RESULTVIEW_3487524704470538157.a4 AS a4, s__RESULTVIEW_3487524704470538157.a5 AS a5, s__RESULTVIEW_3487524704470538157.a6 AS a6, s__RESULTVIEW_3487524704470538157.a7 AS a7, s__RESULTVIEW_3487524704470538157.a8 AS a8, s__RESULTVIEW__8689322627111648167.a1 AS a9, s__RESULTVIEW_3487524704470538157.prob * s__RESULTVIEW__8689322627111648167.prob AS prob FROM s__RESULTVIEW_3487524704470538157,s__RESULTVIEW__8689322627111648167 WHERE s__RESULTVIEW_3487524704470538157.a2 = s__RESULTVIEW__8689322627111648167.a1;
CREATE VIEW s__RESULTVIEW__6625465567401263443 AS SELECT a7 AS a1, a8 AS a2, prob AS prob FROM s__RESULTVIEW_8636114359657583065;
CREATE VIEW s__RESULTVIEW__678859678987752445 AS SELECT a7 AS a1, a8 AS a2, prob AS prob FROM s__RESULTVIEW_4914011101465958082;
CREATE VIEW s__RESULTVIEW_4646209289709239398 AS SELECT s__RESULTVIEW__6625465567401263443.a1 AS a1, s__RESULTVIEW__6625465567401263443.a2 AS a2, am_all_idf_bm25_unstemmed.termID AS a3, s__RESULTVIEW__6625465567401263443.prob * am_all_idf_bm25_unstemmed.prob AS prob FROM s__RESULTVIEW__6625465567401263443,am_all_idf_bm25_unstemmed WHERE s__RESULTVIEW__6625465567401263443.a1 = am_all_idf_bm25_unstemmed.termID;
CREATE VIEW s__RESULTVIEW_803050599359904103 AS SELECT s__RESULTVIEW__678859678987752445.a1 AS a1, s__RESULTVIEW__678859678987752445.a2 AS a2, gtaa_all_idf_bm25_unstemmed.termID AS a3, s__RESULTVIEW__678859678987752445.prob * gtaa_all_idf_bm25_unstemmed.prob AS prob FROM s__RESULTVIEW__678859678987752445,gtaa_all_idf_bm25_unstemmed WHERE s__RESULTVIEW__678859678987752445.a1 = gtaa_all_idf_bm25_unstemmed.termID;
CREATE VIEW s__RESULTVIEW__1234239311746380292 AS SELECT s__RESULTVIEW_4646209289709239398.a1 AS a1, s__RESULTVIEW_4646209289709239398.a2 AS a2, s__RESULTVIEW_4646209289709239398.a3 AS a3, am_all_termdict_unstemmed.termID AS a4, am_all_termdict_unstemmed.term AS a5, am_all_termdict_unstemmed.sb AS a6, s__RESULTVIEW_4646209289709239398.prob * am_all_termdict_unstemmed.prob AS prob FROM s__RESULTVIEW_4646209289709239398,am_all_termdict_unstemmed WHERE s__RESULTVIEW_4646209289709239398.a1 = am_all_termdict_unstemmed.termID;
CREATE VIEW s__RESULTVIEW_3125223437396834298 AS SELECT s__RESULTVIEW_803050599359904103.a1 AS a1, s__RESULTVIEW_803050599359904103.a2 AS a2, s__RESULTVIEW_803050599359904103.a3 AS a3, gtaa_all_termdict_unstemmed.termID AS a4, gtaa_all_termdict_unstemmed.term AS a5, gtaa_all_termdict_unstemmed.sb AS a6, s__RESULTVIEW_803050599359904103.prob * gtaa_all_termdict_unstemmed.prob AS prob FROM s__RESULTVIEW_803050599359904103,gtaa_all_termdict_unstemmed WHERE s__RESULTVIEW_803050599359904103.a1 = gtaa_all_termdict_unstemmed.termID;
CREATE VIEW s__RESULTVIEW_3881768213967656727 AS SELECT a2 AS a1, a5 AS a2, prob AS prob FROM s__RESULTVIEW__1234239311746380292;
CREATE VIEW s__RESULTVIEW__5159286147922968544 AS SELECT a2 AS a1, a5 AS a2, prob AS prob FROM s__RESULTVIEW_3125223437396834298;
CREATE VIEW s__RESULTVIEW__101494595982612010 AS SELECT s__RESULTVIEW_3881768213967656727.a1 AS a1, s__RESULTVIEW_3881768213967656727.a2 AS a2, s__RESULTVIEW__5159286147922968544.a1 AS a3, s__RESULTVIEW__5159286147922968544.a2 AS a4, s__RESULTVIEW_3881768213967656727.prob * s__RESULTVIEW__5159286147922968544.prob AS prob FROM s__RESULTVIEW_3881768213967656727,s__RESULTVIEW__5159286147922968544 WHERE s__RESULTVIEW_3881768213967656727.a2 = s__RESULTVIEW__5159286147922968544.a2;
CREATE VIEW s__RESULTVIEW_8511588532222625888 AS SELECT a1 AS a1, a3 AS a2, sum(prob) AS prob FROM s__RESULTVIEW__101494595982612010 GROUP BY a1, a3;
CREATE VIEW s__RESULTVIEW AS SELECT a1 AS a1, a2 AS a2, prob AS prob FROM (SELECT a1 AS a1, a2 AS a2, prob AS prob FROM (SELECT s__RESULTVIEW_8511588532222625888.a1 AS a1, s__RESULTVIEW_8511588532222625888.a2 AS a2, s__RESULTVIEW_8511588532222625888.prob / tmp_4585563859082064648.prob AS prob FROM s__RESULTVIEW_8511588532222625888,(SELECT max(prob) AS prob FROM s__RESULTVIEW_8511588532222625888) AS tmp_4585563859082064648) AS tmp__8062118767853448021) AS tmp__242866319706049884 WHERE prob >= 0.5;

WITH source AS (
	SELECT a.*, DENSE_RANK() OVER(ORDER BY a1 DESC) as rownum, ROW_NUMBER() OVER(PARTITION BY a1 ORDER BY prob DESC) as groupnum
       FROM s__RESULTVIEW as a)
SELECT t1.pid as d1, t1.class as d1class, t1.json as d1json, t2.pid as d2, t2.class as d2class, t2.json as d2json, 		source.prob
FROM source,
 am_materialized_1 as t1,
 gtaa_materialized_1 as t2
WHERE t1.id=source.a1 AND t2.id=source.a2 AND       source.groupnum <= 5
  AND source.rownum > 0
  AND source.rownum <= 20
ORDER BY source.prob DESC, a1;

